from .main import AvroBaseModel  # noqa: F401 I001
from .mapper import (
    PYDANTIC_INMUTABLE_FIELDS_CLASSES,
    PYDANTIC_LOGICAL_TYPES_FIELDS_CLASSES,
)
from dataclasses_avroschema.fields import mapper

mapper.IMMUTABLE_FIELDS_CLASSES.update(PYDANTIC_INMUTABLE_FIELDS_CLASSES)  # type: ignore
mapper.LOGICAL_TYPES_FIELDS_CLASSES.update(PYDANTIC_LOGICAL_TYPES_FIELDS_CLASSES)  # type: ignore
mapper.ALL_TYPES_FIELD_CLASSES.update(PYDANTIC_INMUTABLE_FIELDS_CLASSES)
mapper.ALL_TYPES_FIELD_CLASSES.update(PYDANTIC_LOGICAL_TYPES_FIELDS_CLASSES)


__all__ = ["AvroBaseModel"]
